#!/usr/bin/python3

import os
from urllib import parse

import gi
gi.require_version('Nemo', '3.0')
from gi.repository import Nemo, GObject

import signal
signal.signal(signal.SIGINT, signal.SIG_DFL)

class ColumnExtension(GObject.GObject, Nemo.ColumnProvider, Nemo.InfoProvider, Nemo.NameAndDescProvider):
	def __init__(self):
		pass

	def get_columns(self):
		return (
			Nemo.Column(name="NemoPython::nesmapper_column",attribute="nesmapper",label="Mapper",description="iNES Mapper Number"),
			Nemo.Column(name="NemoPython::nesprg_column",attribute="nesprg",label="PRG",description="iNES PRG ROM Size"),
			Nemo.Column(name="NemoPython::neschr_column",attribute="neschr",label="CHR",description="iNES CHR ROM Size"),
		)

	def update_file_info(self, file):
		if file.get_uri_scheme() != 'file':
			return

		# Set blank strings for inappropriate file types
		file.add_string_attribute('nesmapper', '')
		file.add_string_attribute('nesprg', '')
		file.add_string_attribute('neschr', '')

		filename = parse.unquote(file.get_uri()[7:])

		if filename[-4:].lower() != '.nes':
			return

		with open(filename, "rb") as f:
			header = f.read(16)

		# Make sure it's actually a .NES ROM
		if len(header) < 16 or header[0:4] != b'NES\x1a':
			return

		nesmapper = '?'
		nesprg = '?'
		neschr = '?'
		# NES 1.0
		if (header[7] & 0x0C) == 0x00:
			nesmapper = "%4d" % (((header[6] & 0xF0) >> 4) | (header[7] & 0xF0))
			nesprg = "%4d KB" % (header[4] * 16)
			neschr = "%4d KB" % (header[5] * 8)

		# NES 2.0
		if (header[7] & 0x0C) == 0x08:
			nesmapper = "%4d.%02d" % (((header[6] & 0xF0) >> 4) | (header[7] & 0xF0) | ((header[8] & 0x0F) << 8), (header[8] & 0xF0) >> 4)
			suffixes = ["B", "KB", "MB", "GB", "TB", "PB", "EB"]
			if (header[9] & 0xF) == 0xF:
				exp = (header[4] >> 2) & 0x3F
				nesprg = "%4d %s" % ((1 << (exp % 10)) * (((header[4] << 1) & 0x6) | 0x1), suffixes[exp // 10])
			else:
				nesprg = "%4d KB" % ((header[4] | ((header[9] & 0xF) << 8)) * 16)

			if (header[9] & 0xF0) == 0xF0:
				exp = (header[5] >> 2) & 0x3F
				neschr = "%4d %s" % ((1 << (exp % 10)) * (((header[5] << 1) & 0x6) | 0x1), suffixes[exp // 10])
			else:
				neschr = "%4d KB" % ((header[5] | ((header[9] & 0xF0) << 4)) * 8)

		file.add_string_attribute('nesmapper', nesmapper)
		file.add_string_attribute('nesprg', nesprg)
		file.add_string_attribute('neschr', neschr)

	def get_name_and_desc(self):
		return [("Nemo NES Columns:::Provides additional columns for the list view")]

